@@ -149,6 +149,8 @@ class FancySchemaItem(SchemaItem):
                 return value
         except Invalid:
             pass
+        except:
+            pass
         return self._validate(value, **kw)
 
     def _validate(self, value, **kw): return value
@@ -206,7 +208,8 @@ class Object(FancySchemaItem):
         if self.polymorphic_registry:
             disc = d.get(self.polymorphic_on, Missing)
             if disc is Missing:
-                disc = d[self.polymorphic_on] = self.managed_class.__name__
+                mm = self.managed_class.__mongometa__
+                disc = d[self.polymorphic_on] = mm.polymorphic_identity
             else:
                 cls = self.polymorphic_registry[disc]
         if cls is None:
@@ -285,20 +288,26 @@ class Array(FancySchemaItem):
         has_errors = False
         if d is None:
             d = []
-        for value in d:
-            try:
-                value = self.field_type.validate(value)
-                result.append(value)
-                error_list.append(None)
-            except Invalid, inv:
-                error_list.append(inv)
-                has_errors = True
-        if has_errors:
-            msg = '\n'.join(('[%s]:%s' % (i,v))
-                            for i,v in enumerate(error_list)
-                            if v)
-            raise Invalid(msg, d, None, error_list=error_list)
-        return result
+        try:
+            for value in d:
+                try:
+                    value = self.field_type.validate(value)
+                    result.append(value)
+                    error_list.append(None)
+                except Invalid, inv:
+                    error_list.append(inv)
+                    has_errors = True
+            if has_errors:
+                msg = '\n'.join(('[%s]:%s' % (i,v))
+                                for i,v in enumerate(error_list)
+                                if v)
+                raise Invalid(msg, d, None, error_list=error_list)
+            return result
+        except Invalid:
+            raise
+        except TypeError, ex:
+            raise Invalid(str(ex), d, None)
+        
 
 class Scalar(FancySchemaItem):
     '''Validate that a value is NOT an array or dict'''
@@ -354,9 +363,25 @@ class DateTime(ParticularScalar):
     type=datetime
 class Bool(ParticularScalar):
     type=bool
-class ObjectId(ParticularScalar):
-    if_missing=Missing
-    type=pymongo.bson.ObjectId
+class Binary(ParticularScalar):
+    type=pymongo.bson.Binary
+class ObjectId(Scalar):
+    def if_missing(self):
+        return pymongo.bson.ObjectId()
+    def _validate(self, value):
+        try:
+            value = Scalar._validate(self, value)
+            if value is None: return value
+            if isinstance(value, pymongo.bson.ObjectId):
+                return value
+            elif isinstance(value, basestring):
+                return pymongo.bson.ObjectId(str(value))
+            else:
+                raise Invalid('%s is not a bson.ObjectId' % value, value, None)
+        except Invalid:
+            raise
+        except Exception, ex:
+            raise Invalid(str(ex), value, None)
 
 # Shorthand for various SchemaItems
 SHORTHAND={
