@@ -1,6 +1,9 @@
+from __future__ import absolute_import
 import pymongo
+from threading import local
 
-from base import Cursor, Object
+from .base import Cursor, Object
+from . import exc
 
 class Session(object):
     _registry = {}
@@ -19,9 +22,13 @@ class Session(object):
 
     def _impl(self, cls):
         try:
-            return self.bind.db[cls.__mongometa__.name]
+            return self.db[cls.__mongometa__.name]
         except TypeError:
-            return None
+            raise exc.MongoGone, 'MongoDB is not connected'
+
+    @property
+    def db(self):
+        return self.bind.db
 
     def get(self, cls, **kwargs):
         bson = self._impl(cls).find_one(kwargs)
@@ -72,10 +79,11 @@ class Session(object):
         doc.update(data)
         if args:
             values = dict((arg, data[arg]) for arg in args)
-            result = self._impl(doc).update(dict(_id=doc._id), {'$set':values}, safe=True)
+            result = self._impl(doc).update(
+                dict(_id=doc._id), {'$set':values}, safe=True)
         else:
             result = self._impl(doc).save(data, safe=True)
-        if result:
+        if result and '_id' not in doc:
             doc._id = result
 
     def insert(self, doc):
@@ -86,8 +94,8 @@ class Session(object):
             data = dict(doc)
         doc.update(data)
         bson = self._impl(doc).insert(data, safe=True)
-        if isinstance(bson, pymongo.objectid.ObjectId):
-            doc.update(_id=bson)
+        if bson and '_id' not in doc:
+            doc._id = bson
 
     def upsert(self, doc, spec_fields):
         doc.make_safe()
@@ -116,7 +124,8 @@ class Session(object):
 
     def set(self, doc, fields_values):
         """
-        sets a key/value pairs, and persists those changes to the datastore immediately
+        sets a key/value pairs, and persists those changes to the datastore
+        immediately 
         """
         fields_values = Object.from_bson(fields_values)
         fields_values.make_safe()
